#include <streamer/streamer.h>
#include <identity.h>
#include "common.h"

StreamElementType *_identity;

StreamElement *Stream_identity_create() {
  Stream_identity *element;

  element = (Stream_identity *)malloc(sizeof(Stream_identity));

  element->element.flags = 0;
  element->element.inpadfunc = NULL; /* lave it up to the general case */
  element->element.outpadfunc = NULL; /* ditto */
  element->element.chain = Stream_identity_chain;
  element->element.push = NULL;
  element->element.inpads = NULL;
  element->element.outpads = NULL;

  /* construct the input pad */
  element->in.type = 0;
  element->in.flags = 0;
  element->in.owner = (StreamElement *)element;
  element->in.peer = NULL;
  element->in.chain = Stream_identity_chain;
  DEBUG("identity set chain to %p\n",element->in.chain);
  /* register it */
  element->element.inpads =
    g_slist_append(element->element.inpads,&element->in);

  element->out.type = 0;
  element->out.flags = 0;
  element->out.owner = (StreamElement *)element;
  element->out.peer = NULL;
  element->out.chain = NULL;
  /* register it */
  element->element.outpads =
    g_slist_append(element->element.outpads,&element->out);

  return (StreamElement *)element;
}

void Stream_identity_chain(StreamPad *pad,StreamBuffer *buf) {
  Stream_identity *identity_element = (Stream_identity *)pad->owner;
  if (identity_element->out.chain) {
    DEBUG("identity about to chain buffer %p, contents '%s', %d \
bytes, to %p()\n",buf,buf->data,buf->datasize,identity_element->out.chain);
    (identity_element->out.chain)(identity_element->out.peer,buf);
  }
}


void init_StreamPlugin(StreamPlugin *plugin) {
  /* we have to do this because we're going to return static data */
  g_module_make_resident(plugin->module);

  /* fill in the StreamPlugin data */
  plugin->name = "identity";
  plugin->longname = "Identity filters";
  plugin->version = "0.0.0";
  plugin->elementtypes = NULL;
  plugin->enumtypes = NULL; /* simple case, we rely on default */

  /* now create the element type */
  _identity = (StreamElementType *)g_malloc(sizeof(StreamElementType));
  _identity->plugin = plugin;
  _identity->name = "identity";
  _identity->longname = "Identity";
  _identity->version = "0.0.0";
  _identity->create = Stream_identity_create;

  /* add it to the necessary lists */
  plugin->elementtypes = g_slist_append(plugin->elementtypes,_identity);
  StreamElementType_register(_identity);

  /* announce that we're here... */
  DEBUG("initialized plugin '%s'\n",StreamPlugin_getname(plugin));
}
