/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * Portions copyright (c) 1995 Brown University.
 * Copyright (c) 1995 The Regents of the University of California.
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

/* 
 * util32.c --
 *
 *      Miscellaneous functions that deal with 32 bit color displays.
 *
 */

#include "global.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "video.h"
#include "gdith.h"



/*
 *--------------------------------------------------------------
 *
 * FindFullColorVisual
 *
 *  Returns a pointer to a full color bit visual on the display
 *
 * Results:
 *      See above.
 *  
 * Side effects:
 *      Unknown.
 *
 *--------------------------------------------------------------
 */
Visual *
FindFullColorVisual (dpy, depth)
     Display *dpy;
     int *depth;
{
  XVisualInfo vinfo;
  XVisualInfo *vinfo_ret;
  int numitems, maxdepth;
  DBUG_ENTER();
  
  vinfo.class = TrueColor;
  
  vinfo_ret = XGetVisualInfo(dpy, VisualClassMask, &vinfo, &numitems);
  
  if (numitems == 0) DBUG_RETURN(NULL);

  maxdepth = 0;
  while(numitems > 0) {
    if (vinfo_ret[numitems-1].depth > maxdepth) {
      maxdepth = vinfo_ret[numitems-1 ].depth;
    }
    numitems--;
  }
  XFree((void *) vinfo_ret);

  if (maxdepth < 16) DBUG_RETURN(NULL);

  if (XMatchVisualInfo(dpy, DefaultScreen(dpy), maxdepth, 
		       TrueColor, &vinfo)) {
    *depth = maxdepth;
    DBUG_RETURN(vinfo.visual);
  }
  
  DBUG_RETURN(NULL);
}


/*
 *--------------------------------------------------------------
 *
 * CreateFullColorWindow
 *
 *  Creates a window capable of handling 32 bit color.
 *
 * Results:
 *      See above.
 *  
 * Side effects:
 *      Unknown.
 *
 *--------------------------------------------------------------
 */
void
CreateFullColorWindow (xinfo)
     XInfo *xinfo;
{
  int depth;
  Visual *visual;
  XSetWindowAttributes xswa;
  unsigned long mask;
  unsigned int class;
  int screen;
  Display *dpy=xinfo->display;
  int x = xinfo->hints.x,
      y = xinfo->hints.y;
  unsigned int w = xinfo->hints.width,
               h = xinfo->hints.height;
  DBUG_ENTER();

  screen = XDefaultScreen(dpy);
  class = InputOutput;	/* Could be InputOnly */
  if (xinfo->visual == NULL) {
    xinfo->visual = visual = FindFullColorVisual (dpy, &depth);
    xinfo->depth = depth;
  } else {
     visual=xinfo->visual;
     depth=xinfo->depth;
  }

  if (visual == NULL) {
    DBUG_VOID_RETURN();
  }
  mask = CWBackPixel | CWColormap | CWBorderPixel;
  if (xinfo->cmap==0) {
  xswa.colormap = XCreateColormap(dpy,
				  XRootWindow(dpy, screen),
                                  visual, AllocNone);
  } else xswa.colormap = xinfo->cmap;
  xswa.background_pixel = BlackPixel(dpy, DefaultScreen(dpy));
  xswa.border_pixel = WhitePixel(dpy, DefaultScreen(dpy));

  xinfo->window = XCreateWindow(dpy, RootWindow(dpy, screen), x, y, w, h,
    (unsigned int) 1, depth, class, visual, mask, &xswa);
  DBUG_VOID_RETURN();
}
