#include "streamer-private.h"

GSList *_StreamBufferPool_list;

void _StreamBufferPool_init() {
  DEBUG("in _StreamBufferPool_init()\n");
  _StreamBufferPool_list = NULL;
}

StreamBufferPool *StreamBufferPool_create(gint buffers,glong typemetasize,
                                          glong metasize,glong datasize) {
  StreamBufferPool *pool;
  glong totalsize;

  /* insanity check */
  if (buffers <= 0) {
    g_warning("StreamBufferPool_create: buffers not greater than 0!\n");
    return(NULL);
  }

  /* see how much we need to allocate per buffer */
  totalsize = sizeof(StreamBuffer) + typemetasize + metasize + datasize;

  /* allocate the pool */
  pool = (StreamBufferPool *)g_malloc(sizeof(StreamBufferPool));
  pool->mutex = g_mutex_new();
  pool->buffers = 0;
  pool->typemetasize = typemetasize;
  pool->metasize = metasize;
  pool->datasize = datasize;

  /* initial pool */
  pool->size = buffers;
  pool->used = 0;
  pool->chunk = g_mem_chunk_new("StreamBuffer pool",totalsize,
                                totalsize*pool->size,G_ALLOC_AND_FREE);
  /* auxilliary pool for slow resize */
  pool->newchunk = NULL;
  pool->newsize = 0;
  pool->newused = 0;
  /* list to extras */
  pool->extra = NULL;

  /* add it to the global list of pool */
  _StreamBufferPool_list = g_slist_append(_StreamBufferPool_list,pool);

  return(pool);
}

void StreamBufferPool_resize(StreamBufferPool *pool, gint newsize) {
  /* insanity check */
  if (!pool) {
    g_warning("StreamBufferPool_resize: pool is NULL!\n");
    return;
  }

  /* we can't resize if there's already a resize in progress */
  if (pool->newchunk) {
    g_warning("StreamBufferPool_resize: pool is already being resized\n");
    return;
  }

  /* this is actually rather simple... */
  pool->newsize = newsize;
  pool->newused = 0;
  pool->newchunk = g_mem_chunk_create(StreamBuffer,pool->newsize,
                                      G_ALLOC_AND_FREE);
}

gboolean StreamBufferPool_isresizing(StreamBufferPool *pool) {
  /* insanity check */
  if (!pool) {
    g_warning("StreamBufferPool_resize: pool is NULL!\n");
    return;
  }

  if (pool->newchunk) return(TRUE);
  else return(FALSE);
}

gint StreamBufferPool_getsize(StreamBufferPool *pool) {
  /* insanity check */
  if (!pool) {
    g_warning("StreamBufferPool_resize: pool is NULL!\n");
    return;
  }

  if (pool->newchunk) return(pool->newsize);
  return(pool->size);
}

gint StreamBufferPool_getbufcount(StreamBufferPool *pool) {
  /* insanity check */
  if (!pool) {
    g_warning("StreamBufferPool_resize: pool is NULL!\n");
    return;
  }

  return(pool->buffers);
}

void StreamBufferPool_autoresize(StreamBufferPool *pool) {
  gint newsize;

  /* insanity check */
  if (!pool) {
    g_warning("StreamBufferPool_resize: pool is NULL!\n");
    return;
  }

  /* we can't do anything if it's already being resized */
  if (pool->newchunk) return;

  /* see if there's a need to resize */
  if (pool->buffers > pool->size) {
    newsize = pool->size;
    while (newsize < pool->buffers) newsize *= 2;
    DEBUG("StreamBufferPool_autoresize: setting pool to %d buffers\n",
          newsize);
    StreamBufferPool_resize(pool,newsize);
  }
}
