#ifndef STREAMER_ELEMENT_H
#define STREAMER_ELEMENT_H

#include <streamer/streamer.h>

struct _StreamPad {
  /* what type of data is being fed through? */
  glong type;
  glong flags;

  /* who owns this pad */
  StreamElement *owner;  
  gint padnumber;

  /* who's the peer? */
  StreamPad *peer;

  /* chaining function */
  void (*chain)(StreamPad *,StreamBuffer *);
};

struct _StreamElementType {
  StreamPlugin *plugin;

  gchar *name;
  gchar *longname;
  gchar *version;

  /* element creation function */
  StreamElement *(*create)();
};

struct _StreamElement {
  StreamElementType *type;
  glong flags;

  /* pads */
  GSList *inpads, *outpads;

  /* utility functions */
  StreamPad *(*inpadfunc)(StreamElement *,int);
  StreamPad *(*outpadfunc)(StreamElement *,int);
  void (*chain)(StreamPad *,StreamBuffer *);
  void (*push)(StreamElement *);
};

void StreamElementType_register(StreamElementType *type);
StreamPad *StreamElement_inpad(StreamElement *element,int idx);
StreamPad *StreamElement_outpad(StreamElement *element,int idx);
void StreamElement_connect(StreamElement *out,int idxout,
                           StreamElement *in,int idxin);
void StreamElement_push(StreamElement *element);

#endif /* STREAMER_ELEMENT_H */
